' Add a Reference to System.Data.OracleClient
Imports System.Data.Common

'
' A "database agnostic" class for updating the underlying database.  It accepts
' any of the .Net supported database "providers"
'

Public Class Database
    Implements IDisposable

    Private TableList() As String

#If V7 Then
    ' Whew... Talk about late binding!
    Private da As Object
    Private con As Object
    Private cmd As Object
    Private cb As Object
#Else
    Private da As Common.DbDataAdapter
    Private con As Common.DbConnection
    Private cmd As Common.DbCommand
    Private cb As Common.DbCommandBuilder
#End If

    Public Sub New()
        Dim Provider, Connect As String
        Dim c As New Crypt

        ' Since referential integrity is turned on, we must fill the tables
        ' the correct order.  Consequently, this is a list of tables in the
        ' order we want them.
        TableList = New String() {"SOS", "Profiles", "Shares", "Accounts", "Admins", "AutoUpdate", "BIOS", "Components", "CPU", "Desktop", "Devices", "Disks", "Drives", "Email", "EventLogs", "EventLogSettings", "FileInfo", "Internet", "Mapped", "Memory", "Modem", "Monitor", "Motherboard", "Mouse", "NetAdapter", "NetConfig", "OS", "PC", "Permissions", "Ports", "Printers", "Processes", "QFE", "Registry", "Services", "Software", "Startup", "SystemDrivers", "SystemInfo", "Tasks", "Video", "Virus", "UserLogin"}

        Provider = My.Settings.DatabaseProvider
        If My.Settings.EncryptConnectString Then
            Connect = c.Decrypt(My.Settings.ConnectionString)
        Else
            Connect = My.Settings.ConnectionString
        End If

        ' a quick sanity check.  Bad encryption keys result in an empty
        ' connection string
        If Provider <> "None" And Connect = "" Then
            Throw New ApplicationException("Invalid Connection string.  The Connection string" & vbCr & "was empty or decryption failed")
        End If

        con = Nothing
        cmd = Nothing
        da = Nothing
        cb = Nothing
        Select Case Provider
            Case "None"
                'if no dataprovider is configured, then just exit silently
                Exit Sub
            Case "System.Data.Odbc"
                con = New Odbc.OdbcConnection(Connect)
                cmd = New Odbc.OdbcCommand
                da = New Odbc.OdbcDataAdapter
                cb = New Odbc.OdbcCommandBuilder(DirectCast(da, Odbc.OdbcDataAdapter))
            Case "System.Data.OleDb"
                con = New OleDb.OleDbConnection(Connect)
                cmd = New OleDb.OleDbCommand
                da = New OleDb.OleDbDataAdapter
                cb = New OleDb.OleDbCommandBuilder(DirectCast(da, OleDb.OleDbDataAdapter))
            Case "System.Data.OracleClient"
                con = New OracleClient.OracleConnection(Connect)
                cmd = New OracleClient.OracleCommand
                da = New OracleClient.OracleDataAdapter
                cb = New OracleClient.OracleCommandBuilder(DirectCast(da, OracleClient.OracleDataAdapter))
            Case "System.Data.SqlClient"
                con = New SqlClient.SqlConnection(Connect)
                cmd = New SqlClient.SqlCommand
                da = New SqlClient.SqlDataAdapter
                cb = New SqlClient.SqlCommandBuilder(DirectCast(da, SqlClient.SqlDataAdapter))
            Case Else
                Throw New ApplicationException(Provider & " Database provider is not supported")
        End Select

        ' As the tables get larger, the possibility of a timeout increases
        cmd.CommandTimeout = 90
    End Sub

    '
    ' Query the underlying database for a complete set of records
    '
    Public Sub QueryData(ByVal dsTemp As SOSOSDataSet, ByVal PC_Name As String, ByVal PCorID As String, ByVal isID As Boolean)
        Dim TableName, CurrentTable, ID As String
        Dim dt As DataTable
        Dim DidOpen As Boolean = False

        ' a bit of sanity checking before we start...
        If IsNothing(con) OrElse IsNothing(dsTemp) Then
            Exit Sub
        End If

        ' CurrentTable is used in the Error reporting to show what module
        ' was running when the error occurred.
        CurrentTable = "QueryData"
        Try
            If con.State <> ConnectionState.Open Then
                con.Open()
                DidOpen = True
            End If
            cmd.Connection = con
            da.SelectCommand = cmd

            ' whack any existing records in the temp database
            If dsTemp.SOS.Count > 0 Then
                dsTemp.Clear()
            End If

            ID = ""
            ' Did we pass a database "ID" field or just the name of the PC?
            If isID = False Then
                ' query the SOS table to get the ID field
                cmd.CommandText = "Select ID from SOS where PC_Name='" & PCorID & "'"
                ' if there are multiple matches, this returns the first one.
                ID = CType(cmd.ExecuteScalar, String)
                If IsNothing(ID) Then
                    ' there are no matching records, so there's nothing to do
                    Exit Sub
                End If
            Else
                ID = PCorID
            End If

            ' load the tables in the correct order
            For Each TableName In TableList
                CurrentTable = TableName
                dt = dsTemp.Tables(TableName)

                If TableName = "UserLogin" Then
                    ' The UserLogin table.  This is a bit different... here we
                    ' retrieve all of the historical records for that PC.
                    cmd.CommandText = "Select * from UserLogin Where ID='" & ID & "' order by 2"
                Else
                    cmd.CommandText = "Select * from " & dt.TableName & " where " & dt.Columns(0).ColumnName & "='" & ID & "' order by 2"
                End If
                Try
                    da.Fill(dsTemp, dt.TableName)
                Catch ex As Exception
                    ' This is a special case for the extra security on
                    ' the EventLog table.  We typically remove select
                    ' permissions from that table for a vast number of
                    ' users.  This is not an error, it's an expected
                    ' condition
                    If CurrentTable <> "EventLogs" Then
                        ' if the error occurs in some table *other* than
                        ' Eventlogs, then it's a real error
                        Throw
                    End If
                End Try
            Next
        Catch ex As Exception
            ' Record this error and also let it "bubble up" to the next
            ' error handler
            Misc.ErrorLog(ex, PC_Name, CurrentTable, Misc.ErrLogLevel.Errors)
            Throw
        Finally
            If con.State <> ConnectionState.Closed And DidOpen Then
                con.Close()
            End If
        End Try

    End Sub

    '
    ' Execute a query that returns a list of PC Names (used in Poll_byAge)
    '
    Public Function GetPCList(ByVal sql As String) As String()

        If IsNothing(con) Then
            Return Nothing
        End If

        Dim reader As IDataReader
        Dim sc As New System.Collections.Specialized.StringCollection
        Dim ans() As String

        con.Open()
        cmd.Connection = con
        cmd.CommandText = sql
        reader = cmd.ExecuteReader

        While reader.Read()
            sc.Add(reader.GetString(0))
        End While
        con.Close()

        If sc.Count > 0 Then
            ReDim ans(sc.Count - 1)
            sc.CopyTo(ans, 0)
            Return ans
        End If

        Return Nothing
    End Function

    '
    ' Fill a DataTable based upon the SQL select clause
    '
    Public Function Fill(ByVal sql As String, ByVal dt As DataTable) As Integer
        ' sanity check
        If IsNothing(con) Then
            Return 0
        End If

        cmd.Connection = con
        cmd.CommandText = sql
        da.SelectCommand = cmd
        Return da.Fill(dt)
    End Function

    '
    ' Save the data to the underlying database
    '
    Public Sub RecordData(ByVal ds As SOSOSDataSet, ByVal PC_Name As String)
        Dim dt As DataTable
        Dim ReverseList() As String
        Dim TableName, ID, CurrentTable As String

        CurrentTable = "RecordData"

        ' a bit of sanity checking before we start...
        If IsNothing(con) OrElse IsNothing(ds) OrElse ds.SOS.Count = 0 Then
            Exit Sub
        End If

        Try
            ID = ds.SOS(0).ID

            ' open the database connection
            con.Open()
            cmd.Connection = con
            da.SelectCommand = cmd

            ' make a "reverse" TableList array
            ReDim ReverseList(TableList.Length - 1)
            Array.Copy(TableList, ReverseList, TableList.Length)
            Array.Reverse(ReverseList)

            ' Remove all of the existing records for this PC from the
            ' underlying database (in reverse order)
            For Each TableName In ReverseList
                CurrentTable = TableName
                If TableName <> "UserLogin" Then
                    dt = ds.Tables(TableName)
                    cmd.CommandText = "Delete from " & TableName & " where " & dt.Columns(0).ColumnName & "='" & ID & "'"
                    cmd.ExecuteNonQuery()
                End If
            Next

            ' Now add them all back!
            For Each TableName In TableList
                CurrentTable = TableName
                dt = ds.Tables(TableName)
                If dt.Rows.Count > 0 Then
                    ' build a "prototype" select statement so that the
                    ' CommandBuilder will create the insert statement
                    If TableName = "UserLogin" Then
                        cmd.CommandText = "Select * from UserLogin Where UserID='0'"
                    Else
                        cmd.CommandText = "Select * from " & TableName & " where " & dt.Columns(0).ColumnName & "='0'"
                    End If
                    cb.RefreshSchema()
                    Try
                        da.Update(dt)
                    Catch update_ex As Exception
                        ' non-fatal error
                        Misc.ErrorLog(update_ex, PC_Name, CurrentTable, Misc.ErrLogLevel.Errors)
                    End Try
                End If
            Next
        Catch ex As Exception
            ' Record fatal errors and also let them "bubble up" to the next
            ' error handler
            Misc.ErrorLog(ex, PC_Name, CurrentTable, Misc.ErrLogLevel.Errors)
            Throw
        Finally
            If con.State <> ConnectionState.Closed Then
                con.Close()
            End If
        End Try

    End Sub

    '
    ' Just update the UserLogin table
    '
    Public Sub UpdateLoginOnly(ByVal ds As SOSOSDataSet, ByVal PC_Name As String)
        Dim CurrentTable As String

        ' A quick sanity check
        If ds.UserLogin.Rows.Count = 0 Then
            Exit Sub
        End If

        CurrentTable = "UserLogin"
        Try
            con.Open()
            cmd.Connection = con
            da.SelectCommand = cmd
            cmd.CommandText = "Select * from UserLogin Where ID='0'"
            cb.RefreshSchema()

            da.Update(ds.Tables("UserLogin"))
        Catch login_ex As Exception
            Misc.ErrorLog(login_ex, PC_Name, CurrentTable, Misc.ErrLogLevel.Full)
        Finally
            If con.State <> ConnectionState.Closed Then
                con.Close()
            End If
        End Try
    End Sub

    Private disposedValue As Boolean

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                If Not IsNothing(con) Then
                    con.Dispose()
                End If
                If Not IsNothing(cmd) Then
                    cmd.Dispose()
                End If
                If Not IsNothing(da) Then
                    da.Dispose()
                End If
                If Not IsNothing(cb) Then
                    cb.Dispose()
                End If
            End If
        End If
        Me.disposedValue = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

End Class
