Imports System.IO
Imports System.Drawing
Imports System.Windows.Forms
'
' A class to create a "watermark" on the User's existing wallpaper.  The
' watermark is temporary and is replaced upon next login.  It's designed to
' be subtle and not obnoxious.  It uses semi-transparent text on top of 
' whatever wallpaper the user already has.
'
Public Class WallPaper

#Region "API Region"
    Private Const SPI_SETDESKWALLPAPER As Integer = &H14
    Private Const SPIF_UPDATEINIFILE As Integer = &H1
    Private Const SPIF_SENDWININICHANGE As Integer = &H2

    'BOOL SystemParametersInfo(
    '  UINT uiAction,
    '  UINT uiParam,
    '  PVOID pvParam,
    '  UINT fWinIni
    ');
    Private Declare Auto Function SystemParametersInfo Lib "user32.dll" ( _
        ByVal uAction As Integer, _
        ByVal uParam As Integer, _
        ByVal lpvParam As String, _
        ByVal fuWinIni As Integer _
    ) As Integer
#End Region

    Const X_Offset As Integer = 250
    Const Y_Offset As Integer = 5

    ' Note: There is no error checking here, so errors "bubble up" to
    ' the calling routine
    Public Sub MakeWallPaper(ByVal ds As SOSOSDataSet)
        Dim x, y, wp_style, wp_tile As Integer
        Dim rect As Rectangle
        Dim hkcu, reg As RegistryKey
        Dim seed_file, bg_color, status, wp_file As String
        Dim bm_seed, bm_wp As Bitmap
        Dim gr As Graphics
        Dim br_text, br_shadow As SolidBrush
        Dim fnt As Font

        ' a quick sanity check
        If IsNothing(ds) OrElse ds.SOS.Count = 0 Then
            Exit Sub
        End If

        status = WallPaperText(ds)

        ' get the size of the desktop...
        rect = Screen.GetBounds(New Point(1, 1))
        x = rect.Width
        y = rect.Height

        ' get the current wallpaper settings from the registry
        hkcu = Registry.CurrentUser
        reg = hkcu.OpenSubKey("Control Panel\Desktop", False)
        seed_file = reg.GetValue("Wallpaper", "").ToString
        ' one more try...
        If Not File.Exists(seed_file) Then
            seed_file = reg.GetValue("OriginalWallpaper", "").ToString
        End If
        wp_style = CInt(reg.GetValue("WallpaperStyle", 0))
        wp_tile = CInt(reg.GetValue("TileWallpaper", 0))
        reg.Close()

        reg = hkcu.OpenSubKey("Control Panel\Colors", False)
        If IsNothing(reg) Then
            ' for Win9x, use the standard dark green
            bg_color = "0 128 128"
        Else
            bg_color = reg.GetValue("Background", "128 128 128").ToString
            reg.Close()
        End If

        ' load the bitmaps
        bm_seed = Nothing
        If seed_file = "" Or seed_file = "(None)" Then
            wp_style = -1
        Else
            If Not File.Exists(seed_file) Then
                Throw New ApplicationException("Error loading wallpaper seed file '" & seed_file & "'")
            End If
            bm_seed = New Bitmap(seed_file)
        End If
        bm_wp = New Bitmap(x, y)
        gr = Graphics.FromImage(bm_wp)

        ' let's figure out if we should center, tile or stretch.  I don't try to
        ' emulate the pattern feature Win9x/WinNT
        If wp_style = 0 Then
            wp_style += wp_tile
        End If
        Select Case wp_style
            Case -1 ' None
                Dim parts() As String
                Dim r, g, b As Integer
                Dim br As Brush

                parts = bg_color.Split(" "c)
                r = CInt(parts(0))
                g = CInt(parts(1))
                b = CInt(parts(2))
                br = New SolidBrush(System.Drawing.Color.FromArgb(r, g, b))
                gr.FillRectangle(br, 0, 0, x, y)
            Case 0 ' Center
                Dim parts() As String
                Dim r, g, b As Integer
                Dim br As Brush

                parts = bg_color.Split(" "c)
                r = CInt(parts(0))
                g = CInt(parts(1))
                b = CInt(parts(2))
                br = New SolidBrush(System.Drawing.Color.FromArgb(r, g, b))
                gr.FillRectangle(br, 0, 0, x, y)
                gr.DrawImage(bm_seed, (x - bm_seed.Width) \ 2, (y - bm_seed.Height) \ 2, bm_seed.Width, bm_seed.Height)
            Case 1 ' Tile
                Dim tile_x, tile_y As Integer

                ' I could do this with a graphics brush... but I chose to
                ' do multiple DrawImages instead.
                For tile_x = 0 To x Step bm_seed.Width
                    For tile_y = 0 To y Step bm_seed.Height
                        gr.DrawImage(bm_seed, tile_x, tile_y, bm_seed.Width, bm_seed.Height)
                    Next
                Next
            Case 2 ' Stretch
                gr.DrawImage(bm_seed, 0, 0, x, y)
        End Select
        If wp_style >= 0 Then
            bm_seed.Dispose()
        End If

        fnt = New Font("Arial", 10, FontStyle.Regular, GraphicsUnit.Pixel)

        ' Next we build the brushes we plan to use. 50% transparent black for the
        ' drop shadow and 50% transparent white for the text
        br_shadow = New SolidBrush(Color.FromArgb(128, 0, 0, 0))
        br_text = New SolidBrush(Color.FromArgb(128, 255, 255, 255))

        ' write the text (offset the drop shadow by 1 pixel)
        gr.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit
        gr.DrawString(status, fnt, br_shadow, (x - X_Offset) + 1, Y_Offset + 1)
        gr.DrawString(status, fnt, br_text, x - X_Offset, Y_Offset)
        gr.Dispose()

        ' save the resulting bitmap
        wp_file = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData) & "\SOSOS.bmp"
        bm_wp.Save(wp_file, System.Drawing.Imaging.ImageFormat.Bmp)
        bm_wp.Dispose()

        ' Set the wallpaper
        SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, wp_file, SPIF_SENDWININICHANGE Or SPIF_UPDATEINIFILE)

        ' Put the original settings back in the registry. The line above
        ' changes it to wp_file.  However, we want a "fresh" starting point
        ' each time a user log in, so we make it look like the wallpaper
        ' is the orginal seed_file (without the water mark).

        ' Note: this is the only place that requires write permission to the registry
        reg = hkcu.OpenSubKey("Control Panel\Desktop", True)
        reg.SetValue("Wallpaper", seed_file)
        reg.Close()
    End Sub

    '
    ' Make a text summary for the wallpaper
    ' Note: The contents of this summary are only configurable via editing the
    ' source code and recompiling.
    '
    Private Shared Function WallPaperText(ByVal ds As SOSOSDataset) As String
        Dim buf As New System.Text.StringBuilder
        Dim dr As DataRow

        dr = ds.SOS(0)
        ' For simplicity's sake, we're not using the strongly-typed
        ' interface to the DataSet (so we don't have to deal with null
        ' values)
        buf.Append("Name: " & dr("PC_Name").ToString & vbCrLf)
        buf.Append("Domain: " & dr("PC_Domain").ToString & vbCrLf)
        buf.Append("Model: " & dr("PC_Model").ToString & vbCrLf)
        If ds.SystemInfo.Count > 0 Then
            buf.Append("Serial: " & ds.SystemInfo(0)("System_SerialNumber").ToString & vbCrLf)
        End If
        If ds.BIOS.Count > 0 Then
            buf.Append("BIOS_Version: " & ds.BIOS(0)("BIOS_Version").ToString & vbCrLf)
        End If
        buf.Append("User: " & dr("UserName").ToString & vbCrLf)
        buf.Append("IsAdmin: " & dr("IsAdmin").ToString & vbCrLf)
        buf.Append("Comment: " & dr("PC_Comment").ToString & vbCrLf)
        buf.Append("Date: " & dr("CurrentDate").ToString & vbCrLf)
        buf.Append("IP_Address: " & dr("IP_Address").ToString & vbCrLf)
        'buf.Append("MAC_Address: " & dr("MAC_Address").ToString & vbCrLf)
        buf.Append("DHCP: " & dr("DHCP").ToString & vbCrLf)
        buf.Append("OS: " & dr("OS").ToString & vbCrLf)
        buf.Append("CPU_Type: " & dr("CPU_Type").ToString & vbCrLf)
        buf.Append("CPU_Speed: " & dr("CPU_Speed").ToString & vbCrLf)
        'buf.Append("CPU_Count: " & dr("CPU_Count").ToString & vbCrLf)
        buf.Append("Memory: " & dr("Memory").ToString & vbCrLf)
        buf.Append("Disk_Size: " & dr("Disk_Size").ToString & vbCrLf)
        buf.Append("Disk_Free: " & dr("Disk_Free").ToString & vbCrLf)
        'buf.Append("Disk_Count: " & dr("Disk_Count").ToString & vbCrLf)
        buf.Append("Install_Rev: " & dr("Install_Rev").ToString & vbCrLf)
        buf.Append("Install_Date: " & dr("Install_Date").ToString & vbCrLf)
        buf.Append("MSOffice_Version: " & dr("MSOffice_Version").ToString & vbCrLf)
        buf.Append("IE_Version: " & dr("IE_Version").ToString & vbCrLf)
        buf.Append("Virus_Version: " & dr("Virus_Version").ToString & vbCrLf)
        buf.Append("Virus_Defs: " & dr("Virus_Defs").ToString & vbCrLf)
        buf.Append("Virus_Scan: " & dr("Virus_Scan").ToString & vbCrLf)
        'buf.Append("AutoUpdate: " & dr("AutoUpdate").ToString & vbCrLf)
        If ds.AutoUpdate.Count > 0 Then
            buf.Append("LastQFE: " & ds.AutoUpdate(0)("AutoUpdate_LastQFE").ToString & vbCrLf)
            buf.Append("LastQFEDate: " & ds.AutoUpdate(0)("AutoUpdate_LastQFEDate").ToString & vbCrLf)
        End If
        If ds.PC.Count > 0 Then
            buf.Append("LastBoot: " & ds.PC(0)("PC_LastBoot").ToString & vbCrLf)
            'buf.Append("LastLogin: " & ds.PC(0)("PC_LastLogin").ToString & vbCrLf)
        End If

        Return buf.ToString
    End Function
End Class
