Imports System.ComponentModel
Imports System.IO
Imports System.Drawing.Printing

'
' The application's main form.  Has three different viewing "states"
'   Text mode which is compact (and optionally filtered)
'   Grid mode which allows column sorting, etc
'   Report mode which is the most verbose
'
Public Class fmMain
    Private ds As New SOSOSDataSet
    Private RemotePC As String
    Private DoDetails As Boolean        ' DoDetails is a per-user setting
    Private CurrentView As Views        ' CurrentView is a per-user setting
    Private CurrentFont As Font         ' CurrentFont is a per-user setting
    Private TableList As New System.Collections.Specialized.StringCollection
    Private SelectedTable As String
    Private dgv As New DataGridView
    Private tbv As New TextBox

    Private Enum Views As Integer
        TextView = 0
        GridView
        ReportView
    End Enum

    Private Sub GUI_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Dim TestIt As New TestPerm

        ' Let's check out the necessary assembly permissions now, so that we 
        ' don't have any surprises later.  Oddly enough, the program won't even
        ' load if you have a FileDialog control loaded and you don't have the
        ' proper assembly permissions.
        If TestIt.DoTestPerm = False Then
            MsgBox("SOSOS requires significant permissions for it to work properly" & vbCrLf & "Ask your administrator to consult the SOSOS documentation and" & vbCrLf & "adjust the 'Assembly Permissions' or 'Security Zone'", MsgBoxStyle.Critical, "Insufficient Assembly Permissions")
            Me.Close()
        End If

        ' Load the User's default settings
        Try
            DoDetails = My.Settings.DefaultDoDetails
            tsmiDetails.Checked = DoDetails

            SelectedTable = "SOS"
            Select Case My.Settings.DefaultView
                Case "Text"
                    tsmiViewText.PerformClick()
                Case "Grid"
                    tsmiViewGrid.PerformClick()
                Case "Report"
                    tsmiViewReport.PerformClick()
            End Select

            Dim fc As New FontConverter
            CurrentFont = CType(fc.ConvertFromString(My.Settings.DefaultFont), Font)
            Me.Font = CurrentFont
        Catch ex As Exception
            MsgBox("Error parsing the configuration file.  Use the configuration utilities" & vbCr & "provided to make changes to the config files", MsgBoxStyle.Critical, "Configuration Error")
            Me.Close()
        End Try

#If VIEW_ONLY Then
        ' turn off things that are not available in the View-Only mode
        tsmiFileDoit.Enabled = False
        tsmiFileSave.Enabled = False
        tsbDoit.Enabled = False
        tsbSave.Enabled = False
        tstbRemotePC.Enabled = False
        tsmiPoll.Enabled = False
        tsmiSetup.Enabled = False
        tsmiDataWallpaper.Enabled = False
        tslRemote.Enabled = False
        Me.Text &= " (View-Only Mode)"

        ' The View Only interface supports a PC Name as a command-line
        ' argument.  It's used to populate the viewer with historical data
        If My.Application.CommandLineArgs.Count = 1 Then
            tstbFViewOther.Text = My.Application.CommandLineArgs(0)
        Else
            tstbFViewOther.Text = Environment.MachineName
        End If
        tstbFViewOther.PerformClick()
#End If
    End Sub

#If Not VIEW_ONLY Then
    '
    ' Prep things before we start work
    '
    Private Sub tsbDoit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbDoit.Click, tsmiFileDoit.Click
        If ds.SOS.Count > 0 Then
            TreeView1.Nodes.Clear()
            ds.Clear()
        End If

        tspbStatus.Value = 0
        tspbStatus.Maximum = ds.Tables.Count

        ' The list of tables showing in the TreeView
        TableList.Clear()

        TreeView1.Enabled = False
        TreeView1.Nodes.Clear()
        TreeView1.Nodes.Add("")

        ' Since the collection of data is sometimes slow and we don't want the
        ' form to appear "stuck", we use a BackgroundWorker thread to do the
        ' actual data collection.
        Me.Cursor = Cursors.WaitCursor
        BackgroundWorker1.RunWorkerAsync(tstbRemotePC.Text)
    End Sub

    '
    ' Multi-threading keeps the GUI from appearing "frozen"
    '
    Private Sub BackgroundWorker1_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles BackgroundWorker1.ProgressChanged
        Dim TableName As String

        tspbStatus.Increment(e.ProgressPercentage)
        TableName = CType(e.UserState, String)

        If TableName <> "" Then
            TreeView1.Nodes(0).Text = TableName
            TableList.Add(TableName)
        End If

    End Sub

    '
    ' Gather the data (Hey, that's what we're here for!)
    ' Note: the ReportProgress method is used to increment the statusbar
    ' and to provide the table name currently being worked.
    '
    Private Sub BackgroundWorker1_DoWork(ByVal sender As Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles BackgroundWorker1.DoWork
        Dim snoop As Snoop
        Dim worker As BackgroundWorker

        snoop = Nothing
        Try
            worker = CType(sender, BackgroundWorker)
            RemotePC = CType(e.Argument, String)

            If My.Settings.UsePingValidation And RemotePC <> "" Then
                Dim p As New Ping
                If p.Ping(RemotePC) > 1000 Then
                    MsgBox(RemotePC & " Didn't answer a ping", MsgBoxStyle.Information, "Warning")
                    ' this is not an "error", it's an expected condition
                    Exit Sub
                End If
            End If

            worker.ReportProgress(1, "SOS")

            ' start it up...
            Try
                snoop = New Snoop(RemotePC, ds)
            Catch com_ex As System.Runtime.InteropServices.COMException When com_ex.ErrorCode = &H800706BA
                ' not an error condition, so it's not recorded
                MsgBox("The Remote PC is not responding.  It may be off line or" & vbCr & "firewall settings are preventing the connection", MsgBoxStyle.Exclamation, "Error")
                Exit Sub
            Catch ua_ex As System.UnauthorizedAccessException
                ' not normally recorded
                Misc.ErrorLog(ua_ex, RemotePC, "Initialize", Misc.ErrLogLevel.Full)
                MsgBox("The Remote PC refused the connection.  This is most likely" & vbCr & "due to a permissions issue." & vbCr & "Note: Windows XP Home Edition cannot be configured to support remote connections", MsgBoxStyle.Exclamation, "Error")
                Exit Sub
            Catch wmi_ex As System.Management.ManagementException When wmi_ex.ErrorCode = Management.ManagementStatus.AccessDenied
                ' not normally recorded
                Misc.ErrorLog(wmi_ex, RemotePC, "Initialize", Misc.ErrLogLevel.Full)
                MsgBox("You do not have sufficient privileges on the remote PC" & vbCr & "SOSOS requires Administrator rights on the Remote PC", MsgBoxStyle.Exclamation, "Error")
                Exit Sub
            Catch ex As Exception
                ' Record this in the ErrorLog and provide a "generic" error
                ' message to the user
                Misc.ErrorLog(ex, RemotePC, "Initialize", Misc.ErrLogLevel.Errors)
                If IsNothing(ex.InnerException) Then
                    MsgBox("Could not get initial data" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
                Else
                    MsgBox("Could not get initial data" & vbCr & ex.Message & vbCr & ex.InnerException.Message, MsgBoxStyle.Critical, "Error")
                End If
                Exit Sub
            End Try

            If RemotePC = "" Then
                ds.SOS(0).RecordType = "Manual"
            Else
                ds.SOS(0).RecordType = "Remote"
            End If

            ' gather the data...
            If My.Settings.DoAccounts Then
                worker.ReportProgress(1, "Accounts")
                snoop.GetAccounts()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoAdmins Then
                worker.ReportProgress(1, "Admins")
                snoop.GetAdmins()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoAutoUpdate Then
                worker.ReportProgress(1, "AutoUpdate")
                snoop.GetAutoUpdate()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoBIOS Then
                worker.ReportProgress(1, "BIOS")
                snoop.GetBIOS()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoComponents Then
                worker.ReportProgress(1, "Components")
                snoop.GetComponents()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoCPU Then
                worker.ReportProgress(1, "CPU")
                snoop.GetCPU()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoDevices Then
                worker.ReportProgress(1, "Devices")
                snoop.GetDevices()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoDisks Then
                worker.ReportProgress(1, "Disks")
                snoop.GetDisks()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoDrives Then
                worker.ReportProgress(1, "Drives")
                snoop.GetDrives()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoEventLogs Then
                worker.ReportProgress(1, "EventLogSettings")
                snoop.GetEventLogSettings()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoEventLogs Then
                worker.ReportProgress(1, "EventLogs")
                snoop.GetEventLogs()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoFileInfo Then
                worker.ReportProgress(1, "FileInfo")
                snoop.GetFileInfo()
                snoop.GetInstallVer()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoMemory Then
                worker.ReportProgress(1, "Memory")
                snoop.GetMemory()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoModem Then
                worker.ReportProgress(1, "Modem")
                snoop.GetModem()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoMonitor Then
                worker.ReportProgress(1, "Monitor")
                snoop.GetMonitor()
            Else
                worker.ReportProgress(1, "")
            End If

            ' SystemInfo goes before Motherboard
            If My.Settings.DoSystemInfo Then
                worker.ReportProgress(1, "SystemInfo")
                snoop.GetSystemInfo()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoMotherboard Then
                worker.ReportProgress(1, "Motherboard")
                snoop.GetMotherboard()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoMouse Then
                worker.ReportProgress(1, "Mouse")
                snoop.GetMouse()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoNetAdapter Then
                worker.ReportProgress(1, "NetAdapter")
                snoop.GetNetAdapter()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoNetConfig Then
                worker.ReportProgress(1, "NetConfig")
                snoop.GetNetConfig()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoOS Then
                worker.ReportProgress(1, "OS")
                snoop.GetOS()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoPC Then
                worker.ReportProgress(1, "PC")
                snoop.GetPC()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoPorts Then
                worker.ReportProgress(1, "Ports")
                snoop.GetPorts()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoPrinters Then
                worker.ReportProgress(1, "Printers")
                snoop.GetPrinters()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoProcesses Then
                worker.ReportProgress(1, "Processes")
                snoop.GetProcesses()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoQFE Then
                worker.ReportProgress(1, "QFE")
                snoop.GetQFE()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoRegistry Then
                worker.ReportProgress(1, "Registry")
                snoop.GetRegistry()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoServices Then
                worker.ReportProgress(1, "Services")
                snoop.GetServices()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoShares Then
                worker.ReportProgress(1, "Shares")
                snoop.GetShares()
            Else
                worker.ReportProgress(1, "")
            End If

            ' Permissions goes after Shares
            If My.Settings.DoPermissions Then
                worker.ReportProgress(1, "Permissions")
                snoop.GetPermissions()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoSoftware Then
                worker.ReportProgress(1, "Software")
                snoop.GetSoftware()
                snoop.GetSoftwareSummary()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoSystemDrivers Then
                worker.ReportProgress(1, "SystemDrivers")
                snoop.GetSystemDrivers()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoTasks Then
                worker.ReportProgress(1, "Tasks")
                snoop.GetTasks()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoUserLogin Then
                worker.ReportProgress(1, "UserLogin")
                ' We leave this entry here in the TreeView, but we don't populate
                ' it with any data.
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoVideo Then
                worker.ReportProgress(1, "Video")
                snoop.GetVideo()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoVirus Then
                worker.ReportProgress(1, "Virus")
                snoop.GetVirus()
                snoop.GetVirusSummary()
            Else
                worker.ReportProgress(1, "")
            End If

            ' The user profile section...
            If My.Settings.DoProfiles Then
                worker.ReportProgress(1, "Profiles")
                snoop.GetProfiles()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoDesktop Then
                worker.ReportProgress(1, "Desktop")
                snoop.GetDesktop()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoEmail Then
                worker.ReportProgress(1, "Email")
                snoop.GetEmail()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoInternet Then
                worker.ReportProgress(1, "Internet")
                snoop.GetInternet()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoMapped Then
                worker.ReportProgress(1, "Mapped")
                snoop.GetMapped()
            Else
                worker.ReportProgress(1, "")
            End If

            If My.Settings.DoStartup Then
                worker.ReportProgress(1, "Startup")
                snoop.GetStartup()
            Else
                worker.ReportProgress(1, "")
            End If

        Catch ex As Exception
            Misc.ErrorLog(ex, RemotePC, "Internal", Misc.ErrLogLevel.Errors)
            If IsNothing(ex.InnerException) Then
                MsgBox("Internal Error" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
            Else
                MsgBox("Internal Error" & vbCr & ex.Message & vbCr & ex.InnerException.Message, MsgBoxStyle.Critical, "Error")
            End If
            Exit Sub
        Finally
            ' requires an explicit Dispose to unmount the registry hives
            If Not IsNothing(snoop) Then
                snoop.Dispose()
            End If
        End Try
    End Sub

    '
    ' Data gathering is complete, prep the viewing areas
    '
    Private Sub BackgroundWorker1_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles BackgroundWorker1.RunWorkerCompleted
        Dim table, tables() As String

        ' Sort the table names...
        ReDim tables(TableList.Count - 1)
        TableList.CopyTo(tables, 0)
        [Array].Sort(tables)

        TreeView1.Nodes.Clear()
        TreeView1.Nodes.Add("SOS")
        For Each table In tables
            If table <> "SOS" Then
                TreeView1.Nodes.Add(table)
            End If
        Next
        TreeView1.Enabled = True

        SelectedTable = "SOS"
        ShowTable()

        tspbStatus.Value = 0
        Me.Cursor = Cursors.Default
    End Sub

    '
    ' Record the data into the underlying database
    '
    Private Sub tsbSave_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbSave.Click, tsmiFileSave.Click

        ' a quick sanity check...
        If ds.SOS.Count = 0 Then
            MsgBox("There's no data to save", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End If

        If My.Settings.DatabaseProvider = "None" Then
            If MsgBox("There is currently no database configured" & vbCr & "Would you like to configure a database now?", MsgBoxStyle.Information Or MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
                tsmiSetupConfigure.PerformClick()
            End If
            Exit Sub
        End If

        Try
            Application.DoEvents()
            Me.Cursor = Cursors.WaitCursor
            Dim db As New Database
            db.RecordData(ds, RemotePC)
            Me.Cursor = Cursors.Default
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            ' Note: The RecordData method automatically records its errors in
            ' the ErrorLog
            If IsNothing(ex.InnerException) Then
                MsgBox("Could not save the data" & vbCr & ex.Message, MsgBoxStyle.Critical, "Error")
            Else
                MsgBox("Could not save the data" & vbCr & ex.Message & vbCr & ex.InnerException.Message, MsgBoxStyle.Critical, "Error")
            End If
            Exit Sub
        End Try

        MsgBox("Data saved", MsgBoxStyle.Information, "Success")
    End Sub

    Private Sub tstbRemotePC_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles tstbRemotePC.KeyPress
        If e.KeyChar = vbCr Then
            e.Handled = True
            tsbDoit.PerformClick()
        End If
    End Sub

    Private Sub tsmiPollbyAD_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiPollbyAD.Click
        Dim p As New fmPoll_byAD

        p.ShowDialog()
    End Sub

    Private Sub tsmiPollbyBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiPollbyBrowse.Click
        Dim p As New fmPoll_byBrowse

        p.ShowDialog()
    End Sub

    Private Sub tsmiPollbyFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiPollbyFile.Click
        Dim p As New fmPoll_byFile

        p.ShowDialog()
    End Sub

    Private Sub tsmiPollByList_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiPollByList.Click
        Dim p As New fmPoll_byList

        p.ShowDialog()
    End Sub

    Private Sub tsmiPollByIP_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiPollByIP.Click
        Dim p As New fmPoll_byIP

        p.ShowDialog()
    End Sub

    Private Sub tsmiPollByNT_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiPollByNT.Click
        Dim p As New fmPoll_byNT

        p.ShowDialog()
    End Sub

    Private Sub tsmiPollByAge_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiPollByAge.Click
        Dim p As New fmPoll_byAge

        p.ShowDialog()
    End Sub

    '
    ' Creates a text-based summary as a "watermark" on the users wallpaper
    '
    Private Sub tsmiDataWallpaper_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDataWallpaper.Click
        ' a quick sanity check...
        If ds.SOS.Count = 0 Then
            MsgBox("There's no data for wallpaper", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End If

        ' yet another sanity check
        If Environment.MachineName <> ds.SOS(0).PC_Name Then
            MsgBox("You can only make wallpaper with data from your own PC", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End If
        Try
            Dim wp As New WallPaper
            wp.MakeWallPaper(ds)
        Catch ex As Exception
            ' not recorded in the Error log
            MsgBox("Could not create wallpaper" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
        End Try
    End Sub

    '
    ' Change some Configuration setting
    '
    Private Sub tsmiSetupConfigure_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiSetupConfigure.Click
        ' Let's do a quick sanity check to verify that you have write 
        ' permission on the config file

        Dim cfg As System.Configuration.Configuration
        cfg = System.Configuration.ConfigurationManager.OpenExeConfiguration(System.Configuration.ConfigurationUserLevel.None)

        Try
            ' Note: I find it astonishing that Microsoft hasn't adapted
            ' something like the CanWrite method from J# to the rest of the 
            ' .Net Framework platform.  Oh well, we just attempt to open the
            ' file and see if it fails.
            Dim fs As System.IO.FileStream
            fs = System.IO.File.Open(cfg.FilePath, IO.FileMode.Open, IO.FileAccess.ReadWrite)
            fs.Close()
        Catch ex As System.UnauthorizedAccessException
            MsgBox("You don't have permissions to change configuration settings", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End Try

        Dim fmCfg As New fmConfigure
        ' the fmConfigure form supports other applications, so we disable
        ' those parts that don't apply
        Dim c As Control
        For Each c In fmCfg.tpOther.Controls
            c.Enabled = False
        Next
        fmCfg.LoadDefaults()
        fmCfg.ShowDialog()
    End Sub
#Else
    Private Sub View_Only(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbDoit.Click, tsmiFileDoit.Click, tsmiSetupConfigure.Click, tsmiDataWallpaper.Click, tsbSave.Click, tsmiFileSave.Click, tsmiPollbyAD.Click, tsmiPollByAge.Click, tsmiPollbyBrowse.Click, tsmiPollbyFile.Click, tsmiPollByIP.Click, tsmiPollByList.Click, tsmiPollByNT.Click
        MsgBox("Function not supported in the View Only mode", MsgBoxStyle.Information, "View Only")
    End Sub
#End If

    '
    ' The User clicked on an item in the TreeView, so it's time to show
    ' the data in the "right-hand" panel
    '
    Private Sub TreeView1_AfterSelect(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles TreeView1.AfterSelect
        SelectedTable = e.Node.Text
        ShowTable()
    End Sub

    '
    ' Display the table in one of the view modes
    '
    Private Sub ShowTable()
        Select Case CurrentView
            Case Views.GridView
                dgv.DataSource = Nothing
                dgv.DataSource = ds.Tables(SelectedTable)
                ' adjust which columns should be showing
                If dgv.ColumnCount > 0 Then
                    Select Case SelectedTable
                        Case "Components"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Devices"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Disks"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Email"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(2).Visible = False
                        Case "EventLogs"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(2).Visible = False
                        Case "Modem"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Monitor"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Mouse"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "NetAdapter"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "NetConfig"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Permissions"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Printers"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Startup"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(2).Visible = False
                        Case "Tasks"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "UserLogin"
                            ' show all
                        Case "Video"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case "Virus"
                            dgv.Columns(0).Visible = False
                            dgv.Columns(1).Visible = False
                        Case Else
                            dgv.Columns(0).Visible = False
                    End Select
                End If
            Case Views.TextView
                Dim tr As New TextReport(ds)
                Dim sb As New System.Text.StringBuilder

                tr.ReportSection(SelectedTable, sb)
                tbv.Text = sb.ToString
            Case Views.ReportView
                ' find the proper user control to populate the right-hand
                ' panel
                Dim rpv As UserControl
                rpv = Nothing

                Select Case SelectedTable
                    Case "Accounts"
                        rpv = New ucAccounts(ds)
                    Case "Admins"
                        rpv = New ucAdmins(ds)
                    Case "AutoUpdate"
                        rpv = New ucAutoUpdate(ds)
                    Case "BIOS"
                        rpv = New ucBIOS(ds)
                    Case "Components"
                        rpv = New ucComponents(ds)
                    Case "CPU"
                        rpv = New ucCPU(ds)
                    Case "Desktop"
                        rpv = New ucDesktop(ds)
                    Case "Devices"
                        rpv = New ucDevices(ds)
                    Case "Disks"
                        rpv = New ucDisks(ds)
                    Case "Drives"
                        rpv = New ucDrives(ds)
                    Case "Email"
                        rpv = New ucEmail(ds)
                    Case "EventLogs"
                        rpv = New ucEventLogs(ds)
                    Case "EventLogSettings"
                        rpv = New ucEventLogSettings(ds)
                    Case "FileInfo"
                        rpv = New ucFileInfo(ds)
                    Case "Internet"
                        rpv = New ucInternet(ds)
                    Case "Mapped"
                        rpv = New ucMapped(ds)
                    Case "Memory"
                        rpv = New ucMemory(ds)
                    Case "Modem"
                        rpv = New ucModem(ds)
                    Case "Monitor"
                        rpv = New ucMonitor(ds)
                    Case "Motherboard"
                        rpv = New ucMotherboard(ds)
                    Case "Mouse"
                        rpv = New ucMouse(ds)
                    Case "NetAdapter"
                        rpv = New ucNetAdapter(ds)
                    Case "NetConfig"
                        rpv = New ucNetConfig(ds)
                    Case "OS"
                        rpv = New ucOS(ds)
                    Case "PC"
                        rpv = New ucPC(ds)
                    Case "Permissions"
                        rpv = New ucPermissions(ds)
                    Case "Ports"
                        rpv = New ucPorts(ds)
                    Case "Printers"
                        rpv = New ucPrinters(ds)
                    Case "Processes"
                        rpv = New ucProcesses(ds)
                    Case "Profiles"
                        rpv = New ucProfiles(ds)
                    Case "QFE"
                        rpv = New ucQFE(ds)
                    Case "Registry"
                        rpv = New ucRegistry(ds)
                    Case "Services"
                        rpv = New ucServices(ds)
                    Case "Shares"
                        rpv = New ucShares(ds)
                    Case "Software"
                        rpv = New ucSoftware(ds)
                    Case "SOS"
                        rpv = New ucSOS(ds)
                    Case "Startup"
                        rpv = New ucStartup(ds)
                    Case "SystemDrivers"
                        rpv = New ucSystemDrivers(ds)
                    Case "SystemInfo"
                        rpv = New ucSystemInfo(ds)
                    Case "Tasks"
                        rpv = New ucTasks(ds)
                    Case "UserLogin"
                        rpv = New ucUserLogin(ds)
                    Case "Video"
                        rpv = New ucVideo(ds)
                    Case "Virus"
                        rpv = New ucVirus(ds)
                End Select

                SplitContainer1.Panel2.Controls.Clear()
                rpv.Dock = DockStyle.Fill
                SplitContainer1.Panel2.Controls.Add(rpv)
        End Select

    End Sub

    '
    ' Select the Grid View
    ' Note: this is a per-user setting
    '
    Private Sub tsmiViewGrid_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiViewGrid.Click
        tsmiViewText.Checked = False
        tsmiViewReport.Checked = False
        CurrentView = Views.GridView

        SplitContainer1.Panel2.Controls.Clear()
        dgv.Dock = DockStyle.Fill
        dgv.AllowUserToAddRows = False
        dgv.AllowUserToDeleteRows = False
        dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells
        SplitContainer1.Panel2.Controls.Add(dgv)

        My.Settings.DefaultView = "Grid"
        ShowTable()
    End Sub

    '
    ' Select the Text View
    ' Note: this is a per-user setting
    '
    Private Sub tsmiViewText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiViewText.Click
        tsmiViewGrid.Checked = False
        tsmiViewReport.Checked = False
        CurrentView = Views.TextView

        SplitContainer1.Panel2.Controls.Clear()
        tbv.Multiline = True
        tbv.ScrollBars = ScrollBars.Both
        tbv.Dock = DockStyle.Fill
        SplitContainer1.Panel2.Controls.Add(tbv)

        My.Settings.DefaultView = "Text"
        ShowTable()
    End Sub

    '
    ' Select the Report View
    ' Note: this is a per-user setting
    '
    Private Sub tsmiViewReport_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiViewReport.Click
        tsmiViewGrid.Checked = False
        tsmiViewText.Checked = False
        CurrentView = Views.ReportView

        My.Settings.DefaultView = "Report"
        ShowTable()
    End Sub

    '
    ' Import some data from an XML file (hopefully one that we've created!)
    '
    Private Sub tsmiDataImport_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDataImport.Click
        OpenFileDialog1.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*"
        OpenFileDialog1.DefaultExt = ".xml"
        If OpenFileDialog1.ShowDialog() <> Windows.Forms.DialogResult.OK Then
            Exit Sub
        End If

        If ds.SOS.Count > 0 Then
            ' clear out everything
            ds.Clear()
        End If

        Try
            ds.ReadXml(OpenFileDialog1.FileName)
        Catch ex As Exception
            MsgBox("Could not import data from XML file" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
            Exit Sub
        End Try

        ' one little tweak to the data
        ds.SOS(0).RecordType = "Imported"

        ' rebuild the tree view, show the summary table
        BuildTree()
        SelectedTable = "SOS"
        ShowTable()
    End Sub

    '
    ' Export the DataSet to an XML file (without the schema)
    '
    Private Sub tsmiDataExport_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDataExport.Click
        ' a quick sanity check...
        If ds.SOS.Count = 0 Then
            MsgBox("There's no data to export", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End If

        SaveFileDialog1.OverwritePrompt = True
        SaveFileDialog1.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*"
        SaveFileDialog1.FileName = ds.SOS(0).PC_Name & "_" & Now.ToString("yyyyMMdd") & ".xml"
        SaveFileDialog1.DefaultExt = ".xml"
        If SaveFileDialog1.ShowDialog() <> Windows.Forms.DialogResult.OK Then
            Exit Sub
        End If

        Try
            ds.WriteXml(SaveFileDialog1.FileName)
        Catch ex As Exception
            MsgBox("Could not export data to XML file" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
        End Try
    End Sub

    '
    ' Export the DataSet to an XML file (to include the Schema)
    '
    Private Sub tsmiDataExportSchema_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDataExportSchema.Click
        ' a quick sanity check...
        If ds.SOS.Count = 0 Then
            MsgBox("There's no data to export", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End If

        SaveFileDialog1.OverwritePrompt = True
        SaveFileDialog1.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*"
        SaveFileDialog1.FileName = ds.SOS(0).PC_Name & "_" & Now.ToString("yyyyMMdd") & ".xml"
        SaveFileDialog1.DefaultExt = ".xml"
        If SaveFileDialog1.ShowDialog() <> Windows.Forms.DialogResult.OK Then
            Exit Sub
        End If

        Try
            ds.WriteXml(SaveFileDialog1.FileName, XmlWriteMode.WriteSchema)
        Catch ex As Exception
            MsgBox("Could not export data to XML file" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
        End Try
    End Sub

    '
    ' Export the DataSet to a Microsoft Access database file
    '
    Private Sub tsmiDataAccess_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDataAccess.Click
        ' a quick sanity check...
        If ds.SOS.Count = 0 Then
            MsgBox("There's no data to export", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End If

        SaveFileDialog1.Filter = "Microsoft Access files (*.mdb)|*.mdb|All files (*.*)|*.*"
        SaveFileDialog1.FileName = ds.SOS(0).PC_Name & "_" & Now.ToString("yyyyMMdd") & ".mdb"
        SaveFileDialog1.DefaultExt = ".mdb"
        If SaveFileDialog1.ShowDialog() <> Windows.Forms.DialogResult.OK Then
            Exit Sub
        End If

        Try
            Dim dt As DataTable
            Dim mdb As New dt2mdb
            Me.Cursor = Cursors.WaitCursor
            For Each dt In ds.Tables
                mdb.CopyToMDB(dt, SaveFileDialog1.FileName, False)
            Next
            Me.Cursor = Cursors.Default
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            MsgBox("Could not export data to Microsoft Access" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
        End Try
    End Sub

    '
    ' The vanity plate...
    '
    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiHelpAbout.Click
        Dim a As New fmAbout

        a.ShowDialog()
    End Sub

    '
    ' Save the "text report" to a file
    '
    Private Sub tsmiFileSaveText_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiFileSaveText.Click
        ' a quick sanity check...
        If ds.SOS.Count = 0 Then
            MsgBox("There's no data to save", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End If

        SaveFileDialog1.OverwritePrompt = True
        SaveFileDialog1.Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*"
        SaveFileDialog1.FileName = ds.SOS(0).PC_Name & "_" & Now.ToString("yyyyMMdd") & ".txt"
        SaveFileDialog1.DefaultExt = ".txt"
        If SaveFileDialog1.ShowDialog() <> Windows.Forms.DialogResult.OK Then
            Exit Sub
        End If

        Try
            Dim tr As New TextReport(ds)
            Dim buf As String
            buf = tr.PrepareReport(DoDetails)

            Dim fs As FileStream = New FileStream(SaveFileDialog1.FileName, FileMode.Create)
            Dim sw As StreamWriter = New StreamWriter(fs)
            sw.Write(buf)
            sw.Close()
            fs.Close()
        Catch ex As Exception
            MsgBox("Could not save text report" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
        End Try

    End Sub

    '
    ' Should the text report also include a bunch of details?
    '
    Private Sub tsmiDetails_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiDetails.Click
        DoDetails = tsmiDetails.Checked
        My.Settings.DefaultDoDetails = DoDetails
    End Sub

    '
    ' User-selected font sizes
    ' Note: this is a per-user setting
    '
    Private Sub tsmiOptionsFonts_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiOptionsFonts.Click
        FontDialog1.Font = Me.Font
        If FontDialog1.ShowDialog = Windows.Forms.DialogResult.OK Then
            Dim fc As New FontConverter
            Me.Font = FontDialog1.Font
            My.Settings.DefaultFont = fc.ConvertToString(Me.Font)
        End If
    End Sub

    '
    ' Retrieve the records from some other PC
    '
    Private Sub tstbFViewOther_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tstbFViewOther.Click
        ' a quick sanity check...
        If My.Settings.DatabaseProvider = "None" Then
            MsgBox("There is currently no database configured", MsgBoxStyle.Information)
            Exit Sub
        End If

        If tstbFViewOther.Text = "Enter a PC Name" Then
            tstbFViewOther.Text = ""
            Exit Sub
        End If

        Try
            Dim db As New Database
            Dim dsTemp As New SOSOSDataSet
            ' NetBIOS allows for single quote, but I don't
            Dim valid As String = "!@#$%^&()-_{}.~"

            RemotePC = tstbFViewOther.Text
            For Each c As Char In RemotePC.ToCharArray
                If Not Char.IsLetterOrDigit(c) And Not valid.Contains(c.ToString) Then
                    MsgBox("Invalid PC Name", MsgBoxStyle.Exclamation, "Error")
                    Exit Sub
                End If
            Next

            ' pull a query on the data for some PC
            Me.Cursor = Cursors.WaitCursor
            db.QueryData(dsTemp, "", tstbFViewOther.Text, False)

            If dsTemp.SOS.Count = 0 Then
                Me.Cursor = Cursors.Default
                MsgBox("Could not find any records for the PC", MsgBoxStyle.Exclamation, "Warning")
                Exit Sub
            End If

            ds = dsTemp

            ' build the TreeView and show the summary SOS table.
            BuildTree()
            SelectedTable = "SOS"
            ShowTable()
            Me.Cursor = Cursors.Default
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            ' Note: The QueryData method records errors in the ErrorLog
            MsgBox("Could not query the database" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
        End Try

    End Sub

    '
    ' Trap the Enter key while in the "View Other" textbox
    '
    Private Sub tstbFViewOther_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles tstbFViewOther.KeyPress
        ' wait for the Enter key
        If e.KeyChar = vbCr Then
            e.Handled = True
            tstbFViewOther.PerformClick()
        End If
    End Sub

    ' The current tree-node list might not match that of the incoming data.
    ' For example, the current Feature Selection list (which controls which
    ' nodes appear on the TreeView) might not be the same as when the data for
    ' this record was collected.
    Private Sub BuildTree()
        Dim dt As DataTable
        Dim table, tables() As String
        Dim i As Integer

        ' Sort the table names...
        ReDim tables(ds.Tables.Count - 1)
        i = 0
        For Each dt In ds.Tables
            tables(i) = dt.TableName
            i += 1
        Next
        [Array].Sort(tables)

        TreeView1.Nodes.Clear()
        TreeView1.Nodes.Add("SOS")
        For Each table In tables
            If table <> "SOS" Then
                TreeView1.Nodes.Add(table)
            End If
        Next
    End Sub

    '
    ' Retrieve the existing records for this PC
    '
    Private Sub tsmiFViewPC_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiFViewPC.Click

        Try
            Dim db As New Database
            Dim dsTemp As New SOSOSDataSet
            Dim remotePC As String

            ' NetBIOS allows for single quote, but I don't
            Dim valid As String = "!@#$%^&()-_{}.~"

            remotePC = Environment.MachineName
            For Each c As Char In remotePC.ToCharArray
                If Not Char.IsLetterOrDigit(c) And Not valid.Contains(c.ToString) Then
                    MsgBox("Invalid PC Name", MsgBoxStyle.Exclamation, "Error")
                    Exit Sub
                End If
            Next

            ' pull a query on the data for this PC
            Me.Cursor = Cursors.WaitCursor
            db.QueryData(dsTemp, "", Environment.MachineName, False)

            If dsTemp.SOS.Count = 0 Then
                Me.Cursor = Cursors.Default
                MsgBox("Could not find any records for this PC", MsgBoxStyle.Information, "Sorry")
                Exit Sub
            End If

            ds = dsTemp

            ' build the TreeView and show the summary SOS table.
            BuildTree()
            SelectedTable = "SOS"
            ShowTable()
            Me.Cursor = Cursors.Default
        Catch ex As Exception
            Me.Cursor = Cursors.Default
            ' Note: The QueryData method records errors in the ErrorLog
            MsgBox("Could not query the database" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
        End Try
    End Sub

    Private Sub tsmiFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiFileExit.Click
        Me.Close()
    End Sub

    '
    ' Standard printing routines taken directly from the MSDN samples
    '
    Private Sub tsmiFilePrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiFilePrint.Click
        PrintDialog1.Document = PrintDocument1

        ' a quick sanity check...
        If ds.SOS.Count = 0 Then
            MsgBox("There's no data to print", MsgBoxStyle.Exclamation, "Warning")
            Exit Sub
        End If

        Try
            If PrintDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
                Dim tr As New TextReport(ds)
                Dim buf As String
                buf = tr.PrepareReport(DoDetails).Replace(vbTab, "    ")

                streamToPrint = New StringReader(buf)
                Try
                    printFont = New Font("Arial", 10)
                    Dim pd As New PrintDocument()
                    AddHandler pd.PrintPage, AddressOf PrintPage
                    pd.Print()
                Finally
                    streamToPrint.Close()
                End Try
            End If
        Catch ex As Exception
            MsgBox("Could not print the report" & vbCr & ex.Message, MsgBoxStyle.Exclamation, "Error")
        End Try

    End Sub

    Private Sub tsmiFilePrintPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmiFilePrintPreview.Click
        PageSetupDialog1.Document = PrintDocument1
        PageSetupDialog1.ShowDialog()
    End Sub

    Private printFont As Font
    Private streamToPrint As StringReader

    ' The PrintPage event is raised for each page to be printed.
    Private Sub PrintPage(ByVal sender As Object, ByVal ev As PrintPageEventArgs)
        Dim linesPerPage As Single = 0
        Dim yPos As Single = 0
        Dim count As Integer = 0
        Dim leftMargin As Single = ev.MarginBounds.Left
        Dim topMargin As Single = ev.MarginBounds.Top
        Dim line As String = Nothing

        ' Calculate the number of lines per page.
        linesPerPage = ev.MarginBounds.Height / printFont.GetHeight(ev.Graphics)

        ' Print each line of the file.
        While count < linesPerPage
            line = streamToPrint.ReadLine()
            If line Is Nothing Then
                Exit While
            End If
            yPos = topMargin + count * printFont.GetHeight(ev.Graphics)
            ev.Graphics.DrawString(line, printFont, Brushes.Black, leftMargin, yPos, New StringFormat())
            count += 1
        End While

        ' If more lines exist, print another page.
        If Not (line Is Nothing) Then
            ev.HasMorePages = True
        Else
            ev.HasMorePages = False
        End If
    End Sub

    Private Sub tsmi_HelpContents_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmi_HelpContents.Click
        Help.ShowHelp(Me, "SOSOS.chm")
    End Sub

    Private Sub tsmi_HelpIndex_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsmi_HelpIndex.Click
        Help.ShowHelpIndex(Me, "SOSOS.chm")
    End Sub

End Class
